# -*- coding: utf-8 -*-
"""
/***************************************************************************
LandGoed QGIS-NL-base
                                 A QGIS plugin
 SplashScreen and other customizations QGIS-Pro-NL-base
                             -------------------
        begin                : 2019-01-22
        Marco Duiker - LandGoed
        email                : info@landgoed.support

        Adapted from All4GIS/Load-QSS and All4GIS/fake_splash
        Both by Francisco Raga
        
        Adapted from:
        http://www.qgis.nl/2019/02/14/about-layer-tree-embedded-widgets-and-have-your-wmts-always-crispy-sharp/?lang=en
        by Richard Duivenvoorde
 ***************************************************************************/



/***************************************************************************
 *                                                                         *
 *   This program is free software you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation either version 2 of the License, or     *
 #   any later version.                                                    *
 *                                                                         *
 ***************************************************************************/
"""
from __future__ import absolute_import


from builtins import object
import math
import os
import re
import webbrowser

try:
    import sys
    from pydevd import *
except:
    None
    


from qgis.PyQt import QtCore, QtGui
from qgis.PyQt.QtCore import Qt, QCoreApplication, QSettings
from qgis.PyQt.QtWidgets import QWidget, QApplication, QAction, QSplashScreen, \
                                QLabel, QSlider, QCheckBox, QHBoxLayout
from qgis.PyQt.QtGui import QIcon, QPixmap, QCursor

from qgis.core import QgsMapLayer
from qgis.gui import QgsLayerTreeEmbeddedWidgetProvider, \
                     QgsLayerTreeEmbeddedWidgetRegistry

from qgis.core import *
from qgis.gui import *
 
#from .ui import resources_rc
import qgis.utils
from qgis.utils import iface

def reload_style(path):
    ''' reload a custom style. not used for now '''
    # Some applications will remove a file and rewrite it.  QFileSystemWatcher will
    # ignore the file if the file handle goes away so we have to keep adding it.
    watch.removePaths(watch.files())
    watch.addPath(path)
    with open(path, "r") as f:
        stylesheet = f.read()
        # Update the image paths to use full paths. Fixes image loading in styles
        path = os.path.dirname(path).replace("\\", "/")
        stylesheet = re.sub(r'url\((.*?)\)', r'url("{}/\1")'.format(path), stylesheet)
        QApplication.instance().setStyleSheet(stylesheet)

# watch = QFileSystemWatcher()
# watch.fileChanged.connect(reload_style)

class LayerTileScaleWidget(QWidget):
    '''
    A widget to zoom a WMTS, TMS or xyz layer to available zoomlevels.
    This renders the image 'crisp'
    '''

    def __init__(self, layer):
        # layout the widget itself: a label, slider and checkbox
        QWidget.__init__(self)
        label = QLabel("Zoomniveau")
        self.slider = QSlider(Qt.Horizontal)
        self.ckbox = QCheckBox()
        self.ckbox.setToolTip("Zoom alleen naar beschikbare niveau's")
        layout = QHBoxLayout()
        layout.addWidget(label)
        layout.addWidget(self.slider)
        layout.addWidget(self.ckbox)
        self.setLayout(layout)

        self.layer = layer
        self.resolutions = layer.dataProvider().property('resolutions')
        self.mapCanvas = iface.mapCanvas()
        self.slider.setRange(0, len(self.resolutions)-1)
        self.slider.setTickInterval(1)
        self.slider.setTickPosition(QSlider.TicksBothSides)
        # invert because 'highest' value == most zoomed out == 0
        self.slider.setInvertedAppearance(True)
        self.slider.setPageStep(1)
        self.slider.setTracking(False)
        self.slider.setEnabled(True)
        # set the right value of the slider now
        self.on_scale_changed(self.mapCanvas.scale())
        # connect signals
        self.mapCanvas.scaleChanged.connect(self.on_scale_changed)
        self.slider.valueChanged.connect(self.on_value_changed)
        self.ckbox.stateChanged.connect(self.on_chkbox_change)

    def on_chkbox_change(self, state):
        # force a rescaling OR not 
        self.on_scale_changed(self.mapCanvas.scale())

    def on_scale_changed(self, scale):
        if len(self.resolutions)==0:
            return

        try:
            # enable or disabe the 'fixed zooms' checkbox
            self.ckbox.setVisible(self.layer.crs().srsid() == \
                    iface.mapCanvas().mapSettings().destinationCrs().srsid())
            mupp = self.mapCanvas.mapUnitsPerPixel()
            r = 0
            for i in range(0, len(self.resolutions)):
                r = i
                if self.resolutions[i] > mupp:
                    if i > 0 \
                    and (self.resolutions[i]-mupp > mupp-self.resolutions[i-1]):
                        r = i-1
                    break
            # only do fixed zooms if the chkbox is checked AND visible
            if self.ckbox.isVisible() and self.ckbox.isChecked() \
            and not math.isclose(self.resolutions[r], mupp, rel_tol=1e-5):
                self.mapCanvas.zoomByFactor(self.resolutions[r] / \
                    self.mapCanvas.mapUnitsPerPixel())
                return
               
            self.slider.blockSignals(True)
            self.slider.setValue(r)
            self.slider.setToolTip('Z: {:.3f}\nResolution: {:.3f}'.\
                    format(self.slider.maximum() - r, self.resolutions[r]))
            self.slider.blockSignals(False)
        except:
            pass
        

    def on_value_changed(self, value):
        if len(self.resolutions)==0:
            return        
        self.mapCanvas.zoomByFactor(self.resolutions[value] / \
            self.mapCanvas.mapUnitsPerPixel())


class LayerTileScaleWidgetProvider(QgsLayerTreeEmbeddedWidgetProvider):

    def __init__(self):
        QgsLayerTreeEmbeddedWidgetProvider.__init__(self)

    def id(self):
        # will be the ID of our widget
        return "md2_tilescale"

    def name(self):
        # readable name (used in QGIS gui)
        return "Zoomniveau's"

    def createWidget(self, layer, widgetIndex):
        return LayerTileScaleWidget(layer)

    def supportsLayer(self, layer):
        # only XYZ, WMTS and TMS layers. 
        # Hopefully catching these with the following:
        try:
            return layer.RasterLayer \
                and layer.dataProvider().supportsLegendGraphic()
        except:
            return False

provider = LayerTileScaleWidgetProvider()
QgsGui.layerTreeEmbeddedWidgetRegistry().addProvider(provider)

class SplashScreen(object):
    '''
    This misleadingly named class sets QGIS-NL additions like 
    splash screen, icons, window title and help menu.
    '''

    def __init__(self, iface):
 
        self.iface = iface
        self.plugin_dir = os.path.dirname(__file__)
        
        self.windowTitle = 'LandGoed'  # rest of the title comes from the default project.
        
        self.app = QApplication.instance()
        self.QApp = QCoreApplication.instance() 

        if self.QApp == None:
            self.QApp = QApplication(sys.argv)

        self.QApp.startingUp()
        self.QApp.processEvents()
        self.app.startDragTime()
        
        self.iface.initializationCompleted.connect(self.customization)


    def initGui(self):
        
        self.plugin_dir = os.path.dirname(__file__)
        
        # to be on the safe side
        # also in the QGIS.ini settings
        # QSettings().setValue("/qgis/hideSplash", True)
        self.QApp.processEvents()

        # to be on the safe side
        # the icon on the shortcut has a similar effect, but might be 
        # changed by the user
        icon = QIcon(os.path.join(self.plugin_dir, 'images', 'logo.png'))
        self.app.setWindowIcon(icon)
        self.iface.mainWindow().setWindowIcon(icon)

        self.iface.mainWindow().setWindowTitle(self.windowTitle)  
        self.QApp.processEvents()
        
        if not self.iface.mainWindow().isVisible():
            self.splash_pix = QPixmap(os.path.join(self.plugin_dir, 'images', 'splash.png'))
            #self.splash = QSplashScreen(self.splash_pix, QtCore.Qt.WindowStaysOnTopHint)
            self.splash = QSplashScreen(self.splash_pix)
            self.splash.setMask(self.splash_pix.mask())
            self.splash.show()
            self.QApp.processEvents()      
            # self.applyStyle() # for now we won't make a QGIS-NL specific style
            
        self.addHelpMenuItem()
        return
            
    def run(self):
        pass

    def unload(self):
        self.QApp.processEvents()
        self.iface.initializationCompleted.disconnect(self.customization)
        self.helpAction.deleteLater()        
        return

    def customization(self):         
        self.splash.finish(self.iface.mainWindow())
        self.iface.mainWindow().setWindowTitle(self.windowTitle)  
        self.QApp.processEvents()
        # self.applyStyle() # for now we won't make a QGIS-NL specific style
        
    def applyStyle(self):
        ''' apply a custom style. not used for now '''
        path = os.path.abspath(os.path.join(self.plugin_dir,'Interface','stylesheet.qss'))
        watch.removePaths(watch.files())
        reload_style(path)
        
    def openHelp(self):
        webbrowser.open_new("https://qgis-nl.docs.landgoed.support/") 

    def addHelpMenuItem(self):
        if self.iface.firstRightStandardMenu().objectName() == 'mHelpMenu':
            # help menu is in the expected location
            self.helpAction = QAction(QIcon(os.path.join(self.plugin_dir, 'images', 'logo.png')), 
                                 "LandGoed QGIS-NL Help", self.iface.mainWindow())
            
            self.helpAction.triggered.connect(self.openHelp)
            self.helpAction.setWhatsThis("LandGoed QGIS-NL Help")
            
            self.iface.firstRightStandardMenu().addAction(self.helpAction)
